// Use the db.js library, define a schema,
// and listen for events

db.open({
    name: 'database',
    version: 2,
    schema: {
        items: {
            key: {
                keyPath: 'id',
                autoIncrement: true
            },
            indexes: {
                color: { unique: false }
            }
        }
    }
}).done(function(server){

	// The database was successfully opened. We can
	// run transactions using the server varaible

	// Listen for the document ready event, as we will
	// be working with the dom
	
	$(function() {
	
		// Cache some selectors

		var add = $('#add'),
			items = $('#items');
		
		var colors = ['blue', 'green', 'yellow', 'pink'];
	
		// On dom.ready, select all items and update the #items ul
		server.items.query().filter().execute().done(function(results){
			
			if(!results){
				return;
			}
			
			$.each(results, function(){
				createItem(this);
			});
	
		});
		
		// Listen for clicks on the add button
		add.click(function(){
			
			var item = { 
				text: (new Date()).toTimeString(),
				color: colors[ Math.floor( Math.random()*colors.length )]
			};
			
			server.items.add(item).done(function(){
				createItem(item);
			});
			
			// If you wish to update an item:
			// server.items.update({id:123, color:'pink', text:'asdf'});
		});
		
		// When an item is clicked, remove it from the database.
		$('#items').on('click', 'li:not(#add)', function(){
			var item = $(this);
			server.items.remove( item.data('id') ).done(function(){
				item.fadeOut();
			});
		});
		
		function createItem(item){
			var tmp = $('<li><p></p></li>');
	
			tmp.addClass( item.color )
				.data('id', item.id)
				.find('p').text( item.text );
			
			items.prepend(tmp);
		}
		
	});

}).fail(function(error){
	
	console.error("An error occured: ", error);
	
});