/* 
 *  File:       loginscript.js
 *  Author:     Alan Taran
 *  Class:      JavaScript 2
 *  Project:    Midterm
 *  Remarks:    Log In Script
 */


var userAuthKey = "userAuthorized";

/*
 *  document loaded (ready for manipulation)
 *      Determines if authorized user is logged in.
 *      Prevent unauthorized user from accessing detail information.
 *      Page setup or naviagates approprately.
 *      Sets event handlers.
 */
$(document).ready(function() {

    // check to see if user is currently logged in
    if (sessionStorage.getItem(userAuthKey) !== null) {
        userLoggedIn();
        if (window.location.toString().search("order.html") !== -1) {
            setupOrderDetailPage();
        }
    } else if (window.location.toString().search("order.html") !== -1) {
        // prevent unauthorized user from accessing purchase order details
        sessionStorage.setItem("orderNbr", window.location.search.split("?")[1]);
        window.location.replace("login.html");
    }

    // button and keyboard handlers (located on pages with login and/or logout features)
    if ($("#logout").length > 0) {
        $("#logoutbtn").on("click", logOut);
    }
    if ($("#login").length > 0) {
        $("#loginbtn").on("click", logIn);
        $("#username").on("keyup", logInKeyboard);
        $("#password").on("keyup", logInKeyboard);
        if (window.location.toString().search("login.html") !== -1) {
            $("#login").css("top", "120px").css("left", "100px");
        }
    }
}); 
/* end document ready */


/*
 *  logIn
 *      AJAX call to verify user is authorized.
 */
function logIn() {
    var request = new Object();
    request.username = $("#username").val();
    request.password = $("#password").val();
    var url = "http://mm214.com/javascript2/week5/auth.php";
    $.ajax({
        url: url,
        type: "post",
        dataType: "text",
        data: request,
        success: logInResponse,
        error: function(exc) {
            $("#loginerror").html("Error occurred: " + exc.message);
            console.log("Error trying to log in: " + exc.message);
        }
    });
}   
/* end logIn */


/* 
 *  logInResponse
 *      Process login AJAX response on successful AJAX request.
 *      Check return data to see if user is authorized.
 *      Retain login state for authorized users.
 */
function logInResponse(data) {
    if (data.toLowerCase().search("success") !== -1) {
        console.log("user succesfully logged in; response: " + data);

        // maintain authorized user login state
        var keyValue = $("#username").val();
        sessionStorage.setItem(userAuthKey, keyValue);

        // redirect to original detail page request if user was redirected to the login page
        if (window.location.toString().search("login.html") !== -1) {
            if (sessionStorage.getItem("orderNbr") !== null) {
                var url = "order.html?" + sessionStorage.getItem("orderNbr");
                window.location.replace(url);
            } else {
                window.location.replace("index.html");
            }
        }
        // user remaining on current page - clearing login fields
        $("#password").val("");
        $("#username").val("");
        userLoggedIn();
    } else {
        $("#loginerror").html("Invalid Username and/or Password").show();
        console.log("log in failed; response: " + data);
    }
}   
/* end logInResponse */


/*
 *  userLoggedIn
 *      Page setup for authroized users
 */
function userLoggedIn() {
    if ($("#login").length > 0) {
        $("#login").hide();
    }
    if ($("#logout").length > 0) {
        $("#userid").html(sessionStorage.getItem(userAuthKey));
        $("#logout").show();
    }
}   
/* end userLoggedIn */


/*
 *  logOut
 *      Logout User
 */
function logOut() {
    // remove user login status
    sessionStorage.removeItem(userAuthKey);
    console.log("user logged out");
    
    $("#logout").hide();
    if ($("#login").length > 0) {
        $("#login").show();
    } else {
        window.location.replace("index.html");
    }
}   
/* end logout */


/*
 *  logInKeyboard
 *      Login keyboard event handler. 
 *      Log In button click requires a username and password.
 *      Enter key to automate button click event handler.
 */
function logInKeyboard(evt) {
    if ($("#loginbtn").prop("disabled") === false) {
        if (evt.keyCode === 13) {
            logIn();
        }
    }
    if ($("#username").val().length > 0 && $("#password").val().length > 0) {
        $("#loginbtn").attr("disabled", false);
    } else if ($("#loginbtn").prop("disabled") === false) {
        $("#loginbtn").attr("disabled", true);
    }
}   
/* end logInKeybard */


/*
 *  setupOrderDetailPage
 *      Simulate selected puchase order detail page content
 */
function setupOrderDetailPage() {
    var orderNbr = window.location.search.split("?")[1];
    document.title = "Order " + orderNbr;
    $("#ordernumber").html(orderNbr);
    switch (orderNbr) {
        case "123001" :
            $("#date").html("1/12/2016");
            $("#amt").html("$1,200.00");
            break;
        case "123051" :
            $("#date").html("1/28/2016");
            $("#amt").html("$875.00");
            break;
        case "123255" :
            $("#date").html("2/12/2016");
            $("#amt").html("$920.00");
            break;
    }
}   
/* end setupOrderDetailPage */