// Create the application module
var myApp = angular.module('userApp', []);

// Create the UserCtrl controller
myApp.controller('UserCtrl', ['$scope', '$http', function ($scope, $http) {

    // Models (i.e. data)
    $scope.users = [];
    $scope.newUser = {};

    $scope.newUser.contactTime = _getCurrentContactTime();

    // Use $http service to load the data
    $http.get('data/users.json').success(function (data) {
        $scope.users = data;
        _setIndexes();
    });

    $scope.addUser = function () {
        // Note: $scope.newUser is set through two-way data binding with
        // the new user form in the view
        var newUser = $scope.newUser;
        newUser.state = "normal";
        newUser.index = $scope.users.length;
        $scope.users.push(newUser);
        $scope.newUser = {};
        $scope.newUser.contactTime = _getCurrentContactTime();
    };

    $scope.deleteUser = function (user) {
        if (user.state == "deleted") {
            $scope.users.splice(user.index, 1);
            _setIndexes();
        } else {
            user.state = "deleted";
        }
    };

    $scope.undoDelete = function (user) {
        user.state = "normal";
    };

    $scope.getContactTime = function(user) {
        if (user.contactTime === "12to8") {
            return "12am-8am";
        }

        if (user.contactTime === "8to5") {
            return "8am-5pm";
        }

        if (user.contactTime === "5to12") {
            return "5pm-11:59pm";
        }

        return "";
    };

    /*
     DO EDIT USER
     */
    $scope.editUser = function (user) {
        user.oldName = user.name;
        user.oldAddress = user.address;
        user.oldPhoneNumber = user.phone_number;
        user.oldGender = user.gender;
        user.oldHiking = user.hiking;
        user.oldCooking = user.cooking;
        user.oldReading = user.reading;
        user.oldContactTime = user.contactTime;
        user.state = "edit";
    };

    $scope.saveUser = function (user) {
        // Probably have some Ajax post or patch request
        user.state = "normal";
    };

    $scope.cancelEdit = function (user) {
        user.name = user.oldName;
        user.address = user.oldAddress;
        user.phone_number = user.oldPhoneNumber;
        user.gender = user.oldGender;
        user.hiking = user.oldHiking;
        user.cooking = user.oldCooking;
        user.reading = user.oldReading;
        user.contactTime = user.oldContactTime;
        user.state = "normal";
    };

    // Internal Methods
    function _setIndexes() {
        $scope.users.forEach(function (user, index) {
            user.index = index;
        });
    }

    function _getCurrentContactTime() {
        var hours = new Date().getHours();

        if (hours > 0 && hours < 8) {
            return "12to8";
        }

        if (hours > 8 && hours < 17) {
            return "8to5";
        }

        return "5to12";
    }
}]);