// Create the application module
var myApp = angular.module('userApp', []);

// Create the UserCtrl controller
myApp.controller('UserCtrl', ['$scope', '$http', function($scope, $http) {

  $scope.getCurrentTimeRange = function() {
    var datetime = new Date();
    
    if (0 <= datetime.getHours() && datetime.getHours() < 8) {
      return 'Midnight to 8am';
    }
    else if (8 <= datetime.getHours() && datetime.getHours() < 17) {
      return '8am – 5pm';
    }
    else if (17 <= datetime.getHours() && datetime.getHours() <= 23) {
      return '5pm - 11:59pm';
    }
    
    return '';
  };

  // Models (i.e. data)
  $scope.users = [];
  $scope.newUser = {
    'hobbies': [
      { 'hobby': 'cooking', 'enable': false },
      { 'hobby': 'running', 'enable': false },
      { 'hobby': 'biking', 'enable': false }
    ],
    'time': $scope.getCurrentTimeRange()
  };

  // Use $http service to load the data
  $http.get('data/users.json').success(function(data) {
    $scope.users = data;
    _setIndexes();
  });
  
  $scope.addUser = function() {
    // Note: $scope.newUser is set through two-way data binding with
    // the new user form in the view
    var newUser = $scope.newUser;
    newUser.state = "normal";
    newUser.index = $scope.users.length;
    $scope.users.push(newUser);
    $scope.newUser = {
      'hobbies': [
        { 'hobby': 'cooking', 'enable': false },
        { 'hobby': 'running', 'enable': false },
        { 'hobby': 'biking', 'enable': false }
      ]
    };
  };

  $scope.deleteUser = function(user) {
    if (user.state == "deleted") {
      $scope.users.splice(user.index, 1);
      _setIndexes();
    } else {
      user.state = "deleted";
    }
  };

  $scope.undoDelete = function(user) {
    user.state = "normal";
  };

  /*
   * DO EDIT USER
   */
  $scope.editUser = function(user) {
    user.oldName = user.name;
    user.oldAddress = user.address;
    user.oldPhoneNumber = user.phone_number;

    user.state = "edit";
  };

  $scope.saveUser = function(user) {
    // Probably have some Ajax post or patch request
    user.state = "normal";
  };

  $scope.cancelEdit = function(user) {
    user.name = user.oldName;
    user.address = user.oldAddress;
    user.phone_number = user.oldPhoneNumber;
    user.state = "normal";
  };

  // Internal Methods
  function _setIndexes() {
    $scope.users.forEach(function(user, index) {
      user.index = index;
    });
  }

}]);
