// Create the application module
var myApp = angular.module('userApp', []);

// Create the UserCtrl controller
myApp.controller('UserCtrl', ['$scope', '$http', function($scope, $http) {

  // Models (i.e. data)
  $scope.users = [];
  $scope.newUser = {};
  _setLists();


  // Use $http service to load the data
  $http.get('data/users.json').success(function(data) {
    $scope.users = data;
    _setIndexes();
  });


  $scope.addUser = function() {

    // Note: $scope.newUser is set through two-way data binding with
    // the new user form in the view
    var newUser = $scope.newUser;
    newUser.state = "normal";
    newUser.index = $scope.users.length;
    $scope.users.push(newUser);
    $scope.newUser = {};
    _setLists();
  };

  $scope.deleteUser = function(user) {
    if (user.state == "deleted") {
      $scope.users.splice(user.index, 1);
      _setIndexes();
    } else {
      user.state = "deleted";
    }
  };

  $scope.undoDelete = function(user) {
    user.state = "normal";
  };

 /*
  DO EDIT USER
  */
  $scope.editUser = function(user) {      
    user.oldIndex = user.index;
    user.oldName = user.name;
    user.oldAddress = user.address;
    user.oldPhoneNumber = user.phone_number;
    user.oldZip = user.zip;
    user.oldGender = user.gender;
    user.oldHobbies = [ 
        { "name": "Hiking", "checked" : false },
        { "name": "Music", "checked" : false },            
        { "name": "Reading", "checked" : false },
        { "name": "Softball", "checked" : false },
    ];            
    for(i = 0; i < user.hobbies.length; i++) {
        if (user.hobbies[i].checked) {
            user.oldHobbies[i].checked = true; 
        }
    }
    user.oldTime = user.time;
    user.oldTimeSelected = user.timeSelected;
    user.state = "edit";    
  };




  $scope.saveUser = function(user) {
    // Probably have some Ajax post or patch request
    for(var ix = 0; ix < user.time.length; ix++) {            
        if (user.time[ix].value === user.timeSelected) {
            user.time[ix].selected = true;
        } else {
            user.time[ix].selected = false;
        }            
    }    
    user.state = "normal";
  };


  $scope.cancelEdit = function(user) {
    user.index = user.oldIndex;
    user.name = user.oldName;
    user.address = user.oldAddress;
    user.phone_number = user.oldPhoneNumber;
    user.zip = user.oldZip;
    user.gender = user.oldGender;
    for(i = 0; i < user.oldHobbies.length; i++) {
        if (user.oldHobbies[i].checked) {
            user.hobbies[i].checked = true; 
        } else {
            user.hobbies[i].checked = false;
        }
    }
    user.time = user.oldTime;
    user.timeSelected = user.oldTimeSelected;
    user.state = "normal";
  };

  
  $scope.selectTimeValue = function() {
    var ix = document.getElementById("timeOpt").selectedIndex;
    $scope.newUser.time[ix].selected = true;
    $scope.newUser.timeSelected = $scope.newUser.time[ix].value;
  }

 

  // Internal Methods
  function _setIndexes() {
    $scope.users.forEach(function(user, index) {
      user.index = index;
      for(var ix = 0; ix < user.time.length; ix++) {
          if (user.time[ix].selected) {
              user.timeSelected = user.time[ix].value;
          }
      }
    });
  }

  function _setLists() {
    $scope.newUser.hobbies = [ 
        { "name": "Hiking", "checked" : false },
        { "name": "Music", "checked" : false },            
        { "name": "Reading", "checked" : false },
        { "name": "Softball", "checked" : false },
    ];
    $scope.newUser.time = [
        { "value": "Midnight to 8am", "name": "12-8am",      "selected": false },
        { "value": "8am - 5pm",       "name": "8-5pm",       "selected": false },
        { "value": "5pm - 11:59pm",   "name": "5pm-11:59pm", "selected": false }
    ];
  }
}]);
