﻿/*
Author: Homero Gonzalez
Date: 03/02/2016
*/

"use strict";

appointmentApp.controller("calendarController",
    ["$scope", "$rootScope", "calendarService",
    function calendarController($scope, $rootScope, calendarService) {

        calendarService.getAppointments()
            .then(
            function (result) {
                // Initialize appointments with a source to later support sources per user.
                var appointments = [{ "events": result.data, "sourceId": 1 }];

                // Update scope variables to display calendar.
                $scope.appointments = appointments;
                $scope.initiallyLoaded = true;

                // Initialize local service cache
                calendarService.setAppointments(appointments);
            },
            function (result) {
                $scope.hasError = true;
                $scope.errorMessage = "Unable to get calendar events. " + result.status + " " + result.statusText;
            });

        var showNewForm = function (event) {
            $scope.editableEvent = {};
            $scope.eventAction = "Create";

            if (event !== undefined) {
                $scope.editableEvent = angular.copy(event);
                $scope.eventAction = "Update";
            }

            var defaultDate = new Date();

            if ($scope.editableEvent.start === undefined) {
                $scope.editableEvent.start = defaultDate;
                $scope.editableEvent.startTime = defaultDate; // Needed to display timepicker at 00:00. Control does not support displaying an empty value.
            } else {
                $scope.editableEvent.start = new Date($scope.editableEvent.start); // Convert to date object so datepicker considers it a valid date
                $scope.editableEvent.startTime = $scope.editableEvent.start;
            }

            if ($scope.editableEvent.end === undefined) {
                $scope.editableEvent.end = defaultDate;
                $scope.editableEvent.endTime = new Date(defaultDate.getTime() + 3600000); // Needed to display timepicker at 00:00. Control does not support displaying an empty value.
            } else {
                $scope.editableEvent.end = new Date($scope.editableEvent.end); // Convert to date object so datepicker considers it a valid date
                $scope.editableEvent.endTime = $scope.editableEvent.end;
            }

            // Used to control the visibility of the datepicker popup per field.
            $scope.editableEvent.OpenStatusStart = { opened: false };
            $scope.editableEvent.OpenStatusEnd = { opened: false };

            $scope.displayNewForm = true;
        };

        var eventClick = function (event, element) {
            // Show data entry Form.
            showNewForm(event);
        };

        $scope.checkState = function () {
            console.log("Updated: " + $scope.appointments);
        };

        var mergeDateAndTime = function (date, time) {
            if (date != null) {
                var datetime = new Date(time);
                var year = date.getFullYear();
                var month = date.getMonth();
                var day = date.getDate();
                var hour = datetime.getHours();
                var minute = datetime.getMinutes();
                return new Date(year, month, day, hour, minute, 0);
            }

            return null;
        }

        var validForm = function () {
            if ($scope.newEventForm.$invalid) {
                $scope.submitAttempted = true;
                return false;
            }

            return true;
        }

        $scope.upsertEvent = function () {
            if (!validForm()) {
                return;
            }

            var newEvent = {
                title: $scope.editableEvent.title,
                start: mergeDateAndTime($scope.editableEvent.start,
                    $scope.editableEvent.startTime),
                end: mergeDateAndTime($scope.editableEvent.end,
                    $scope.editableEvent.endTime)
            };

            calendarService.upsertEvent(newEvent);
            $scope.displayNewForm = false;
        }

        $scope.deleteEvent = function () {
            calendarService.deleteEvent($scope.editableEvent.id);
            $scope.displayNewForm = false;
        };

        $scope.showNewForm = showNewForm;

        $scope.cancelNewForm = function () {
            $scope.displayNewForm = false;
        };

        var closeAllCalendars = function () {
            $scope.editableEvent.OpenStatusStart.opened = false;
            $scope.editableEvent.OpenStatusEnd.opened = false;
        }

        $scope.openStartDate = function ($event) {
            $event.preventDefault();
            $event.stopPropagation();

            closeAllCalendars();
            $scope.editableEvent.OpenStatusStart.opened = true;
        };

        $scope.openEndDate = function ($event) {
            $event.preventDefault();
            $event.stopPropagation();

            closeAllCalendars();
            $scope.editableEvent.OpenStatusEnd.opened = true;
        };

        var eventResize = function (event, delta) {
            calendarService.resizeEvent(event, delta);
        };

        $scope.calendarOptions = {
            height: 450,
            editable: true,
            header: {
                left: "month agendaWeek agendaDay",
                center: "title",
                right: "today prev,next"
            },
            timezone: "local", // Important to show events using local time zone even when events are stored as UTC.
            defaultView: "agendaWeek",
            eventResize: eventResize,
            eventClick: eventClick
            // dayClick: $scope.alertEventOnClick,
            // eventDrop: $scope.alertOnDrop,
        };
    }]);