﻿/*
Author: Homero Gonzalez
Date: 03/02/2016
*/

"use strict";

appointmentApp.factory("calendarService",
[
    "$rootScope", "Hub", "$http", "$q",
    function ($rootScope, Hub, $http, $q) {
        var appointments;

        // SignalR Upsert Event Callback.
        var upsertCalendarEvent = function (calendarEvent, sourceId) {
            if (calendarEvent.id === undefined || calendarEvent.id === "" || calendarEvent.id === 0) {
                return; // Invalid input.
            }

            // Verify if event already exists. Note it checks all sources.
            var eventIndex = -1;
            var eventSourceIndex = 0;
            var events = [];
            for (eventSourceIndex; eventSourceIndex < appointments.length; eventSourceIndex++) {
                events = appointments[eventSourceIndex].events;
                eventIndex = events.map(function (x) { return x.id; }).indexOf(calendarEvent.id);

                if (eventIndex !== -1) {
                    break; // EventId was found no need to search more sources.
                }
            }

            // Convert to local date time.
            calendarEvent.start = new Date(calendarEvent.start);
            calendarEvent.end = new Date(calendarEvent.end);
            calendarEvent.stick = true;

            if (eventIndex !== -1) {
                // Update.
                $rootScope.$apply(function () {
                    events[eventIndex] = calendarEvent;
                });
            } else {
                if (sourceId < 1 || sourceId === undefined) {
                    return; // Invalid input for insert.
                }

                eventSourceIndex = appointments.map(function (x) { return x.sourceId; }).indexOf(sourceId);

                if (eventSourceIndex === -1) {
                    return;
                }

                // Insert.
                events = appointments[eventSourceIndex].events;

                $rootScope.$apply(function () {
                    events.push(calendarEvent);
                });
            }
        };

        // SignalR Delete Event Callback.
        var deleteCalendarEvent = function (eventId) {
            if (eventId === undefined || eventId === "" || eventId === 0) {
                return; // Invalid input.
            }

            // Get source and event index.
            var eventIndex = -1;
            var eventSourceIndex = 0;
            var events = [];
            for (eventSourceIndex; eventSourceIndex < appointments.length; eventSourceIndex++) {
                events = appointments[eventSourceIndex].events;
                eventIndex = events.map(function (x) { return x.id; }).indexOf(eventId);

                if (eventIndex !== -1) {
                    break; // EventId was found no need to search more sources.
                }
            }

            if (eventIndex === -1) {
                return; // EventId was not found in any source.
            }

            // Remove event.
            $rootScope.$apply(function () {
                events.splice(eventIndex, 1);
            });
        };

        //Hub setup
        var hub = new Hub("calendarHub", {
            listeners: { // Client callbacks.
                "upsertCalendarEvent": upsertCalendarEvent,
                "deleteCalendarEvent": deleteCalendarEvent
            },
            methods: [ // Server Methods.
                "upsertEvent",
                "deleteEvent"
            ],
            errorHandler: function (error) {
                console.error(error);
            },
            hubDisconnected: function () {
                if (hub.connection.lastError) {
                    hub.connection.start();
                }
            },
            logging: true
        });

        // Gets events from server.
        var getAppointments = function () {
            if (appointments !== undefined) {
                var deferred = $q.defer();
                deferred.resolve(appointments);
                return deferred.promise;
            }

            var requestUrl = "http://jsiihga.azurewebsites.net/Api/EventApi";
            return $http.get(requestUrl);

        };

        // Updates local event cache.
        var setAppointments = function(input) {
            appointments = input;
        };

        // Calls upsert event on the server.
        var upsertEvent = function (event) {
            var eventSourceId = 1; // When we support a source per user we need to receive this as a parameter.

            // Call the UpsertEvent method on the hub.
            hub.upsertEvent(event, eventSourceId);
        };

        // Calls delete event on the server.
        var deleteEvent = function (eventId) {
            // Call the DeleteEvent method on the hub.
            hub.deleteEvent(eventId);
        };

        // Identifies resize changes and calls upsert event on the server.
        var resizeEvent = function (event) {
            // Need to create new event that match server method event type
            var newEvent = {
                id: event.id,
                title: event.title,
                start: event.start,
                end: event.end
            };

            // Call the UpsertEvent method on the hub.
            hub.upsertEvent(newEvent, event.sourceId);
        };

        return {
            getAppointments: getAppointments,
            setAppointments: setAppointments,
            upsertEvent: upsertEvent,
            deleteEvent: deleteEvent,
            resizeEvent: resizeEvent
        }
    }]);