<?php
/* 
 *  Author:     Alan Taran
 *  Class:      JavaScript 2
 *  Project:    Final Project
 *  Date:       March 2016
 *  Remarks:    database class for storing/retrieving Heat Map user activity
 *              useractivity table:
 *              - event         'click'       mouse click
 *                              'move'        mousemove
 *              - grid          'nav'         activity was in top right navigation area
 *                              'ftr'         activity was in the centered footer area
 *                              'pg'          activity was not in navigation or footer areas
 *              - X             X coordinate  from left side of page
 *              - Y             Y coordinate  from top of page
 *              - duration      click event:  number of times users clicked on an area
 *                              move event:   time duration mouse was positioned in an area
 *              X/Y coordiates  a single value is based on a 15/15 pixel area
 *  
 *              Note:   php logic based on projects I did in the PHP class
 */


class HeatmapDB {

    private $dbName = 'sqlite:heatmap_db.sqlite';
    private $userName = '';
    private $password = '';
    
    public function connect2db() {
        try {
            $con = new PDO($this->dbName, $this->userName, $this->password);
        } catch (PDOException $e) {
            echo "Error connecting to database: ".$e->getMessage();
            die();
        }   
        return $con;
    }
    
    
    public function createTable() {
        $con = $this->connect2db();
        $query = "create table if not exists useractivity(event varchar(5) NOT NULL,
                                                          grid varchar(3) NOT NULL,
                                                          X int(5) NOT NULL,
                                                          Y int(5) NOT NULL,
                                                          duration int NOT NULL, 
                                                          PRIMARY KEY (event, grid, X, Y)
                                                          )";
        $statement = $con->prepare($query);
        try {
            $statement->execute();          
        } catch (PDOException $e) {
            echo "Error creating products table: ".$e->getMessage();
            die();
        }
    }

    
    public function getActivity() {
        $con = $this->connect2db();
        $query = "select * from useractivity order by event, duration";
        $statement = $con->prepare($query);
        $statement->execute();
        $userActivity = $statement->fetchAll();
        $statement->closeCursor();
        return $userActivity;
    }
    
    
    public function addActivity($event, $grid, $x, $y, $duration) {
        $con = $this->connect2db();
        // see if entry currently exists
        $query = "select count(*) from useractivity 
                   where event = ?
                     and grid = ?
                     and X = ?
                     and Y = ?";
        $statement = $con->prepare($query);
        $statement->bindValue(1,$event);
        $statement->bindValue(2,$grid);       
        $statement->bindValue(3,$x);       
        $statement->bindValue(4,$y);
        
        $statement->execute();
        $total = $statement->fetchColumn();
        $statement->closeCursor();

        $returnMsg = "success";
        if ($total > 0) {
            // get current duration value
            $query = "select duration from useractivity 
                       where event = ?
                         and grid = ?
                         and X = ?
                         and Y = ?";
            $statement = $con->prepare($query);
            $statement->bindValue(1,$event);
            $statement->bindValue(2,$grid);       
            $statement->bindValue(3,$x);       
            $statement->bindValue(4,$y);
            $statement->execute();
            $curDur = $statement->fetchColumn() ;
            $statement->closeCursor();
            
            // update duration with new activity
            $timeDur = (int)$duration + (int)$curDur;
            $query = "update useractivity
                         set duration = $timeDur
                       where event = ?
                         and grid = ?
                         and X = ?
                         and Y = ?";
            $statement = $con->prepare($query);
            $statement->bindValue(1,$event);
            $statement->bindValue(2,$grid);       
            $statement->bindValue(3,$x);       
            $statement->bindValue(4,$y);    
            $statement->execute();
            try {
                $statement->execute(); 
            } catch (PDOException $e) {
                $returnMsg = "Update failed: ".$e->getMessage();
            }
        } else {
            // add new activity
            $query = "insert into useractivity (event, grid, X, Y, duration) 
                                        values (?,?,?,?,?)";
            $statement = $con->prepare($query);
            $statement->bindValue(1,$event);
            $statement->bindValue(2,$grid);       
            $statement->bindValue(3,$x);       
            $statement->bindValue(4,$y);
            $statement->bindValue(5,$duration);
            try {
                $statement->execute();     
            } catch (PDOException $e) {
                $returnMsg = "Insert failed: ".$e->getMessage();
            }
        }
        $statement->closeCursor();
        
        return $returnMsg;
    }      
}

?>